package CH.psi.midas;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.util.TooManyListenersException;
import javax.swing.*;

public class MJTextField extends javax.swing.JTextField implements OdbKeyUpdate,Serializable
{

  public MJTextField()
  {
    //{{INIT_CONTROLS
	setLayout(null);
	setSize(72,24);
	//}}
  }

  public MJTextField(String odbPath) {
    this();
    setOdbPath(odbPath);
  }
  
  public MJTextField(String odbPath, boolean autoUpdate) {
    this();
    setOdbPath(odbPath);
    this.autoUpdate = autoUpdate;
  }

  //{{DECLARE_CONTROLS
  //}}

  public void setData(java.lang.String data) throws IOException,PropertyVetoException
  {
    String oldValue = new String(this.data);
    String newValue = new String(data);
    
    // call veto
    super.fireVetoableChange("data" ,oldValue, newValue);

    this.data = data;
    
    // signal property change
    super.firePropertyChange("data" ,oldValue, newValue);
    
    // propagate change to ODBKey
    if (odbKey != null && autoUpdate)
      odbKey.setText(data);

    // display new text
    setText(data);
  }
  
  public void propagate() throws IOException {
    // if autoUpdate==false, this routine propagates the control contents to the ODB
    odbKey.setText(data);
  }

  public java.lang.String getData()
  {
    return this.data;
  }

  public void setOdbPath(java.lang.String odbPath)
  {
    this.odbPath = odbPath;
  }

  public java.lang.String getOdbPath()
  {
    return this.odbPath;
  }

  public void setAutoUpdate(boolean autoUpdate)
  {
    try {
      if (!this.autoUpdate && autoUpdate)
	    odbKey.hotLink(this);
            	      
      if (this.autoUpdate && !autoUpdate)
	    odbKey.hotUnlink();
	  } catch (Exception e) { System.out.println(e); }
            	      
    this.autoUpdate = autoUpdate;
  }

  public boolean isAutoUpdate()
  {
    return this.autoUpdate;
  }

  public void connect(Experiment exp) throws 
                IOException, TooManyListenersException, OdbKeyNotFoundException
  {
    // create ODBKey and hot link
    odbKey = new OdbKey(exp, odbPath);
    if (autoUpdate)
      odbKey.hotLink(this);
        
    // set current key value
    setText(odbKey.toString());
    this.data = odbKey.toString();

    // add RETURN listener
    this.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        MJTextField t = (MJTextField)e.getSource();
        try {
          // propagate text field contents to ODB key
          setData(t.getText());
                
          // set focus on next component
          javax.swing.FocusManager.getCurrentManager().focusNextComponent(t);
                
        } catch (IOException ioe) { System.out.println(ioe); } 
            catch (PropertyVetoException pve) {
            JOptionPane.showMessageDialog(null, 
                pve.getMessage(),
                "Improper Value" , 
                JOptionPane.ERROR_MESSAGE, 
                null);
            }
      }
    });
    
    // add FOCUS listener
    this.addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) {
        MJTextField t = (MJTextField)e.getComponent();
 
        if (odbKey != null && !e.isTemporary()) {
          try {
            // propagate text field contents to ODB key
            setData(t.getText());

          } catch (IOException ioe) { System.out.println(ioe); } 
            catch (PropertyVetoException pve) {
              JOptionPane.showMessageDialog(null, 
                  pve.getMessage(),
                  "Improper Value" , 
                  JOptionPane.ERROR_MESSAGE, 
                  null);
              t.requestFocus(); // can't make this work...
              }
        }
      }
    });
  }

  public void update(OdbKey key) 
  {
	String oldValue = new String(this.data);
	String newValue = new String(key.toString());
	this.data = key.toString();
	
    // signal property change
    super.firePropertyChange("data" ,oldValue, newValue);
		
    // display new text
    setText(data);
  }

  static public void main(String args[])
  {
    class DriverFrame extends java.awt.Frame {
	  public DriverFrame() {
	    addWindowListener(new java.awt.event.WindowAdapter() {
          public void windowClosing(java.awt.event.WindowEvent event)
		    {
			dispose();	  // free the system resources
			System.exit(0); // close the application
			}
		});
		
		this.setLayout(null);
		this.setSize(72,24);
		this.add(new MJTextField());
	  }
	}

    new DriverFrame().show();
  }

  // bean properties
  protected java.lang.String odbPath = "";
  protected java.lang.String data = "";
  protected boolean autoUpdate = true;
  
  // bean internal key
  protected OdbKey odbKey = null;
}